use tool_helper::{Input, Output};
use std::io::Read;
pub use tool_helper::Error;

pub enum LineFeed {
    Unix,
    Windows
}

pub enum FileType {
    CHeader,
    CSource,
    CPPHeader,
    CPPSource,
}

pub struct Configuration {
    pub file_name:     String,
    pub data_name:     String,
    pub line_feed:     LineFeed,
    pub file_type:     FileType,
}

struct FileVariables<'a> {
    include:  &'a str,
    var_type: &'a str,
}

const UNIX_LINEFEED:    &'static str = "\n";
const WINDOWS_LINEFEED: &'static str = "\r\n";

const   C_VARIABLES: FileVariables = FileVariables{include: "", var_type: "char"};
const CPP_VARIABLES: FileVariables = FileVariables{include: "#include <stdint.h>", var_type: "uint8_t"};

fn write_source_file(input: Input, output: &mut Output, variables: &FileVariables, data_name: String, line_feed: &str) -> Result<(), std::io::Error> {
    write!(output, "{}{}{}", variables.include, line_feed, line_feed)?;
    write!(output, "const {} {}[] = {{{}", variables.var_type, data_name, line_feed)?;

    let mut byte_line_count = 1;
    let mut bytes           = input.bytes();

    if let Some(byte) = bytes.next() {
        write!(output, "\t{:#04X}", byte?)?;

        for byte in bytes {
            write!(output, ", {:#04X}", byte?)?;
            byte_line_count += 1;

            if byte_line_count >= 16 {
                write!(output, "{}\t", line_feed)?;
                byte_line_count = 0;
            }
        }
    }
    write!(output, "{}}};", line_feed)?;
    Ok(())
}

pub fn convert(cfg: Configuration, input: Input, mut output: Output) -> Result<(), Error> {
    let line_feed = {
        match cfg.line_feed {
            LineFeed::Windows => WINDOWS_LINEFEED,
            LineFeed::Unix    => UNIX_LINEFEED
        }
    };
    let source_variables = {
        match cfg.file_type {
            FileType::CSource   => &C_VARIABLES,
            FileType::CPPSource => &CPP_VARIABLES,
            _                   => return Err(Error::new("Not implemented yet".to_owned(), None))
        }
    };

    Error::try_or_new(write_source_file(input, &mut output, source_variables, cfg.data_name, line_feed), None)
}